from __future__ import annotations

from typing import TYPE_CHECKING, Any
import uuid

import schedule

from jetpack.config import _k8s
from jetpack.proto.runtime.v1alpha1 import remote_pb2

if TYPE_CHECKING:
    from jetpack.core._jetpack_function_with_client import JetpackFunctionWithClient
    from jetpack.core._task import Task


class Tracer:
    """
    Base class for Jetpack instrumentation.
    Inspired by https://github.com/python-trio/trio/blob/6754c74eacfad9cc5c92d5c24727a2f3b620624e/docs/source/tutorial/tasks-with-trace.py
    """

    def task_created(self, task: Task) -> None:
        """Called when runtime client creates new task"""

    def result_returned(self, task_id: uuid.UUID, result: remote_pb2.Result) -> None:
        """Called when runtime client has received new result"""

    def scheduled_task_created(self, task: Task) -> None:
        """Called when new scheduled task created"""

    def cronjob_loaded(self, name: str, repeat_pattern: schedule.Job) -> None:
        """Called when a cronjob has been loaded by SDK"""

    def jetpack_function_called(self, func: JetpackFunctionWithClient[Any]) -> None:
        """Called when jetpack function (@jet or @cron) is executed."""


class LocalTracer(Tracer):
    def task_created(self, task: Task) -> None:
        print(f"Task {task.id} created. Symbol name = {task.symbol_name()}")

    def result_returned(self, task_id: uuid.UUID, result: remote_pb2.Result) -> None:
        print(f"Result for task {task_id} returned.")

    def scheduled_task_created(self, task: Task) -> None:
        print(
            f"Task {task.id} scheduled for time = {task.target_time}. Symbol name = {task.symbol_name()}"
        )

    def cronjob_loaded(self, name: str, repeat_pattern: schedule.Job) -> None:
        print(f"Cronjob {name} loaded. Repeat pattern = {repeat_pattern}")

    def jetpack_function_called(self, func: JetpackFunctionWithClient[Any]) -> None:
        print(f'Jetpack function "{func.name()}" was called')


_tracer = Tracer()
_local_tracer = LocalTracer()


# TODO(Landau): Allow custom tracer
def get_tracer() -> Tracer:
    if _k8s.is_in_cluster():
        return _tracer
    return _local_tracer



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: jetpack/proto/runtime/v1alpha1/app.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from jetpack.proto.runtime.v1alpha1 import remote_pb2 as jetpack_dot_proto_dot_runtime_dot_v1alpha1_dot_remote__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='jetpack/proto/runtime/v1alpha1/app.proto',
  package='userapp',
  syntax='proto3',
  serialized_options=b'\n\013com.userappB\010AppProtoP\001Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\242\002\003UXX\252\002\007Userapp\312\002\007Userapp\342\002\023Userapp\\GPBMetadata\352\002\007Userapp',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n(jetpack/proto/runtime/v1alpha1/app.proto\x12\x07userapp\x1a+jetpack/proto/runtime/v1alpha1/remote.proto\"\xac\x01\n\x07UserApp\x12Z\n\x14revision_to_manifest\x18\x01 \x03(\x0b\x32(.userapp.UserApp.RevisionToManifestEntryR\x12revisionToManifest\x1a\x45\n\x17RevisionToManifestEntry\x12\x10\n\x03key\x18\x01 \x01(\x04R\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x87\x01\n\x0b\x63om.userappB\x08\x41ppProtoP\x01Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\xa2\x02\x03UXX\xaa\x02\x07Userapp\xca\x02\x07Userapp\xe2\x02\x13Userapp\\GPBMetadata\xea\x02\x07Userappb\x06proto3'
  ,
  dependencies=[jetpack_dot_proto_dot_runtime_dot_v1alpha1_dot_remote__pb2.DESCRIPTOR,])




_USERAPP_REVISIONTOMANIFESTENTRY = _descriptor.Descriptor(
  name='RevisionToManifestEntry',
  full_name='userapp.UserApp.RevisionToManifestEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='userapp.UserApp.RevisionToManifestEntry.key', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='userapp.UserApp.RevisionToManifestEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=202,
  serialized_end=271,
)

_USERAPP = _descriptor.Descriptor(
  name='UserApp',
  full_name='userapp.UserApp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='revision_to_manifest', full_name='userapp.UserApp.revision_to_manifest', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='revisionToManifest', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_USERAPP_REVISIONTOMANIFESTENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=99,
  serialized_end=271,
)

_USERAPP_REVISIONTOMANIFESTENTRY.containing_type = _USERAPP
_USERAPP.fields_by_name['revision_to_manifest'].message_type = _USERAPP_REVISIONTOMANIFESTENTRY
DESCRIPTOR.message_types_by_name['UserApp'] = _USERAPP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UserApp = _reflection.GeneratedProtocolMessageType('UserApp', (_message.Message,), {

  'RevisionToManifestEntry' : _reflection.GeneratedProtocolMessageType('RevisionToManifestEntry', (_message.Message,), {
    'DESCRIPTOR' : _USERAPP_REVISIONTOMANIFESTENTRY,
    '__module__' : 'jetpack.proto.runtime.v1alpha1.app_pb2'
    # @@protoc_insertion_point(class_scope:userapp.UserApp.RevisionToManifestEntry)
    })
  ,
  'DESCRIPTOR' : _USERAPP,
  '__module__' : 'jetpack.proto.runtime.v1alpha1.app_pb2'
  # @@protoc_insertion_point(class_scope:userapp.UserApp)
  })
_sym_db.RegisterMessage(UserApp)
_sym_db.RegisterMessage(UserApp.RevisionToManifestEntry)


DESCRIPTOR._options = None
_USERAPP_REVISIONTOMANIFESTENTRY._options = None
# @@protoc_insertion_point(module_scope)
